@@ -40,6 +40,10 @@ PARSE_ERROR         =   28
 ACCESS_DENIED       =   29
 UNKNOWN_SOURCE      =   30
 RT_TO_PERM_FAILED   =   31
+IPSET_WITH_TIMEOUT  =   32
+BUILTIN_IPSET       =   33
+ALREADY_SET         =   34
+MISSING_IMPORT      =   35
 
 INVALID_ACTION      =  100
 INVALID_SERVICE     =  101
@@ -75,6 +79,10 @@ INVALID_USER        =  130
 INVALID_UID         =  131
 INVALID_MODULE      =  132
 INVALID_PASSTHROUGH =  133
+INVALID_MAC         =  134
+INVALID_IPSET       =  135
+INVALID_ENTRY       =  136
+INVALID_OPTION      =  137
 
 MISSING_TABLE       =  200
 MISSING_CHAIN       =  201
@@ -94,6 +102,12 @@ import sys
 class FirewallError(Exception):
     def __init__(self, code, msg=None):
         self.code = code
+        if msg is not None:
+            # escape msg if needed
+            try:
+                x = str(msg)
+            except UnicodeEncodeError:
+                msg = unicode(msg).encode("unicode_escape")
         self.msg = msg
 
     def __repr__(self):
